
/*
  GoodFields 08_temps
  Gestion du temps
  Compactage des sorties série (intégration d'une bibliothèque : Streaming)
  GRAFCET
  TEMPORISATION basique
  FRONT MONTANT/DESCENDANT
  
La gestion du temps, et en particulier des temporisations précises et de l'heure et date sont souvent utiles.
Ajouter la librairie standard Time.h :
#include <time.h>

On va également commencer à faire des "sous-programmes" ou "routines", que j'ai l'habitude de placer en fin de programme

Voir l'example dans Time.h pour la mise à l'heure par voie série ...
*/
 
//**********************************************************
// BIBLIOTHEQUES
// incluses dans le package Arduino de base
#include <Time.h>
// ajoutées
#include <Streaming.h>

//**********************************************************
// DECLARATIONS
// entrées-sorties
int led_a = 13;
// temporisations
int tempo_a,tempo_b;
// vitesse de clignotement : maintenant en millisecondes
int tempo_a_reset=1000;
int tempo_b_reset=700;
// gestion du temps
int interval=10;         // base de temps des traitements en ms
int previousSecond;      // secondes de l'horloge
int top_seconde=false;   // top pour traitements à la seconde dans le P/P
int top_minute=false;   // top pour traitements à la minute dans le P/P
unsigned long previousMillis;
// GRAFCETS
int grafcet_a,grafcet_b;

//**********************************************************
// INITIALISATIONS
void setup() {

// mise à jour de l'horloge
// l'horloge interne est au format UNIX, c'est à dire "nombre de secondes depuis le 1/1/1970 à 0h"
// T1262347200  //noon Jan 1 2010  ( secondes depuis le 01/01/1970 )
//  1261440000 sec/40 ans (le décalage donne 10 jours, soit bien 10 années bissextiles)
//       86400 sec/jour
//    31536000 sec/an
// mise à l'heure
  setTime(12,15,00,25,12,13); // 12h15 le 25 déc 2013 
  
// initialize the digital pins
  pinMode(led_a, OUTPUT);     
// ligne série (pour traces)
  Serial.begin(9600);  // ou 9600 selon votre port COM
  Serial << "Serial ok" << endl;
// on arme les grafcets
  grafcet_a = 1;
  grafcet_b = 1;
}
//**********************************************************
// PROGRAMME PRINCIPAL
void loop() {

// gestion du temps
  ger_top_centieme ();

//--------------------------------
// GRAFCET A
switch (grafcet_a) {

  case 1:
  if ( tempo_a == 0 ) {        // si la tempo est terminée,
    digitalWrite(led_a, HIGH);   // on allume la LED
    tempo_a = tempo_a_reset;             // on arme la tempo
    grafcet_a++;               // et on passe à l'étape suivante
  }
  break;      // cette instruction vous fait sauter à la fin du "switch"
  // sinon, on continue, et cela fait perdre du temps, et il peut arriver que d'autre "case" fonctionnent ! 

  case 2:
  if ( tempo_a == 0 ) {
    digitalWrite(led_a, LOW);    // turn the LED off by making the voltage LOW
    tempo_a = tempo_a_reset;             // on arme la tempo
    grafcet_a=1;               // on reboucle
  }    
  break;

// si grafcet_a n'a pas une valeur correcte, on passe ici
  default: 
      grafcet_a = 0; // on arrête
  break;
}      // fin du "switch"
// FIN GRAFCET A
//--------------------------------
// GRAFCET B
switch (grafcet_b) {

  case 1:
  if ( tempo_b == 0 ) {        // si la tempo est terminée,
    horodatage_serial(); Serial << " grafcet_b=" << grafcet_b <<  " grafcet_a=" << grafcet_a <<endl;
    tempo_b = tempo_b_reset;             // on arme la tempo
    grafcet_b++;               // et on passe à l'étape suivante
  }
  break;      // cette instruction vous fait sauter à la fin du "switch"

  case 2:
  if ( tempo_b == 0 ) {
    horodatage_serial(); Serial << " grafcet_b=" << grafcet_b <<  " grafcet_a=" << grafcet_a <<endl;
    tempo_b = tempo_b_reset;             // on arme la tempo
    grafcet_b=1;               // on reboucle
  }    
  break;

// si grafcet_a n'a pas une valeur correcte, on passe ici
  default: 
      grafcet_b = 0; // on arrête
  break;
}      // fin du "switch"
// FIN GRAFCET B
//--------------------------------
}
// ************************************************************************************************
// SOUS-PROGRAMMES
// ************************************************************************************************
// Programme au -ieme de seconde
// gestion des temporisations
// NB : le débordement du compteur en (unsigned long) a lieu tous les 50 jours environ ...
void ger_top_centieme (){
  unsigned long delta;
  unsigned long currentMillis = millis();
  delta = currentMillis - previousMillis;
  if (delta >= interval) {
// save the last time you come here
    previousMillis = currentMillis;   

// scrutation du clavier toutes les 10ms
//    ger_scrute_clavier();

// allumage temporisé led clavier
//    if (lamp_tempo >= delta) { lamp_tempo -= delta; digitalWrite(lamp, HIGH); } else { lamp_tempo = 0; digitalWrite(lamp, LOW); }

// temporisations diverses
//    if (fonction_tempo >= delta) { fonction_tempo -= delta; } else { fonction_tempo = 0; }
// gestion des TEMPORISATIONS
    if (tempo_a >= delta ) { tempo_a -= delta; } else { tempo_a = 0; }
    if (tempo_b >= delta ) { tempo_b -= delta; } else { tempo_b = 0; }
  
// tops seconde et minute pour traitements dans le main
    if (second() != previousSecond) {
      previousSecond = second();
      top_seconde = true;
      if (previousSecond == 0) top_minute=true;
    }
  }
  return;
}
// ************************************************************************************************
// affichage horloge complete sur la sortie serie, sans fin de ligne
void horodatage_serial() {
// digital clock display of the time
//  Serial << day() << "/" << month() << "/" << year() << " " << hour() << printDigits(minute()) << printDigits(second()) << endl; 
  Serial << ((day()<10)?"0":"") << day() << "/" << ((month()<10)?"0":"") << month() << "/" << year() << " " << ((hour()<10)?"0":"") << hour() << ":" << ((minute()<10)?"0":"") << minute() << ":" << ((second()<10)?"0":"") << second();
}
// ************************************************************************************************
/*
// Transforme la date et l'heure "lisible" en format UNIX pour mise à l'heure et date
time_t setupArduinoTime(int y,int m, int j, int h, int mi, int se) {
  tmElements_t tm;
  tm.Year = y-1970;
  tm.Month = m;
  tm.Day = j;
  tm.Hour = h;
  tm.Minute = mi;
  tm.Second = se;
  time_t time = makeTime(tm);
  return time;
}
// voici le s/p original qui provient d'une bibliothèque pour GPS :
time_t gpsTimeToArduinoTime(){
// returns time_t from gps date and time with the given offset hours
  tmElements_t tm;
  int year;
  gps.crack_datetime(&year, &tm.Month, &tm.Day, &tm.Hour, &tm.Minute, &tm.Second, NULL, NULL);
  time_t time = makeTime(tm);
  return time + (offset * SECS_PER_HOUR);
}
*/
// ************************************************************************************************

